/* eslint-disable */
import {
  Http, IDictionary, Router, Metadata, Loader,
} from '@zeedhi/core';
import {
  AlertService, DialogService, Form, LoadingService, Modal, ModalService, Select, Text, TextInput,
} from '@zeedhi/common';
import { TekGrid } from '@zeedhi/teknisa-components-common';
import { Utilities } from '@/util/Utilities';

export class TabAddressController {
  private addAddress!: Modal;

  private editionAddress!: Modal;

  private NRENDERECOPARC: any;

  public BPController = Loader.getInstance('BusinessPartnerController');

  public validate({ component }: any) {
    const nameForm = component.name.split('_')[0];
    this.validateCEP(nameForm);
  }

  public async validateCEP(nameForm: string) {
    const CEP = Metadata.getInstance<TextInput>(`${nameForm}-AddCEP`).value.replace('-', '');

    if (CEP) {
      LoadingService.show();
      (this.BPController.searchCep(CEP) as any).then(async (response: IDictionary) => {
        const addressFound = response.data.dataset.ENDERECO;

        if (addressFound) {
          Metadata.getInstance<TextInput>(`${nameForm}-Address`).value = (addressFound.DSENDERECO) ? addressFound.DSENDERECO : '';
          await Metadata.getInstance<Select>(`${nameForm}-AddCountry`).setValue((addressFound.CDPAIS) ? addressFound.CDPAIS : '');
          await Metadata.getInstance<Select>(`${nameForm}-AddState`).setValue((addressFound.SGESTADO) ? addressFound.SGESTADO : '');
          await Metadata.getInstance<Select>(`${nameForm}-AddCity`).setValue((addressFound.CDMUNICIPIO) ? addressFound.CDMUNICIPIO : '');
          if (addressFound.CDBAIRRO === '') {
            Http.post('/findCodeDistrict', {
              NMBAIRRO: addressFound.NMBAIRRO,
              SGESTADO: addressFound.SGESTADO,
              CDPAIS: addressFound.CDPAIS,
              CDMUNICIPIO: addressFound.CDMUNICIPIO,
            }).then((response: IDictionary) => {
              response.data.dataset.codeFound.forEach((codeFound: IDictionary) => {
                Metadata.getInstance<Select>(`${nameForm}-AddDistrict`).setValue((codeFound.CDBAIRRO) ? codeFound.CDBAIRRO : '');
              });
            });
          }
        }
      }).catch((error: any) => {
          if (error.data) this.BPController.alertMessage(error.data.error, 'error')
          else this.BPController.alertMessage('Erro ao realizar a busca.', 'error');
          Metadata.getInstance<TextInput>(`${nameForm}-AddCEP`).value = '';
          Metadata.getInstance<TextInput>(`${nameForm}-Address`).value =  '';
          Metadata.getInstance<Select>(`${nameForm}-AddCountry`).value = '';
          Metadata.getInstance<Select>(`${nameForm}-AddState`).value = '';
          Metadata.getInstance<Select>(`${nameForm}-AddCity`).value = '';
          Metadata.getInstance<Select>(`${nameForm}-AddDistrict`).value = '';
      }).finally(() => {
        LoadingService.hide();
      });
    }
    await (nameForm === 'formAddAddress') ? this.changeStateToEnabledModal() : this.changeStateToEnabledModalEdition();
  }

  public alertMessages(message: string, type: string) {
    AlertService.show({
      name: 'basic-alert',
      text: message,
      top: true,
      left: false,
      timeout: 3000,
      color: type,
    });
  }

  public dialogMessage(message: string) {
    DialogService.show({
      name: 'warning-dialog',
      type: 'warning',
      title: 'Erro!',
      text: message,
    });
  }

  public showDeleteDialog({ event } : any) {
    event.preventDefault();

    DialogService.show({
      name: 'buttons-dialog',
      type: 'warning',
      title: 'Excluir Endereço',
      text: 'Todas as informações serão perdidas se concluir está ação. Deseja continuar?',
      buttons: [
        {
          name: 'button-no',
          component: 'ZdButton',
          label: 'Não',
          events: {
            click: () => DialogService.hide(),
          },
          outline: true,
        },
        {
          name: 'button-yes',
          component: 'ZdButton',
          label: 'Sim',
          events: {
            click: '{{TabAddressController.deleteAddress}}'
          },
        },
      ],
    });
  }

  public async deleteAddress() {
    LoadingService.show('Deletando...');
    const grid = Metadata.getInstance<TekGrid>('tekGrid-tabAddress');
    const NRS = (grid.selectedRows.map((element: any) => element.NRENDERECOPARC)).toString();
    LoadingService.show();
    await Http.post('/deleteAddress', {
      NRENDERECOPARC: NRS,
      NRPARCNEGOCIO: this.BPController.currentRow.NRPARCNEGOCIO,
    }).then(() => {
        this.alertMessages('Endereço excluído com sucesso', 'success');
        grid.datasource.get();
        grid.selectedRows = [];
        DialogService.hide();
        LoadingService.hide();
    }).catch((error: any) => {
        Utilities.showErrorMessage(error);
        LoadingService.hide();
      });;
  }

  // Funções do Modal de Adição de Endereço
  public async openModalAddAddress() {
    this.addAddress = await ModalService.createFromJson('BusinessPartner/Modal/modal-add-address', true);
    this.addAddress.show();
  }

  public closeModalAddAddress() {
    this.addAddress.hide();
    this.addAddress?.destroy();
  }

  public onMountNameModal() {
    Metadata.getInstance<Text>('nameOfCorporate-addAddress').text = this.BPController.NAMEOFCORPORATE;
  }

  public async changeStateToEnabledModal() {
    const selectCountry = Metadata.getInstance<Select>('formAddAddress-AddCountry');
    const selectState = Metadata.getInstance<Select>('formAddAddress-AddState');

    if (!selectCountry.value) {
      selectState.disabled = true;
      selectState.value = '';
    } else {
      selectState.disabled = false;
      selectState.datasource.setFilter({
        CDPAIS: selectCountry.value,
      });
    }
    await this.changeCityToEnabledModal();
  }

  public async changeCityToEnabledModal() {
    const selectCountry = Metadata.getInstance<Select>('formAddAddress-AddCountry');
    const selectState = Metadata.getInstance<Select>('formAddAddress-AddState');
    const selectCity = Metadata.getInstance<Select>('formAddAddress-AddCity');

    if (!selectState.value) {
      selectCity.disabled = true;
      selectCity.value = '';
    } else {
      selectCity.disabled = false;
      selectCity.datasource.setFilter({
        CDPAIS: selectCountry.value,
        SGESTADO: selectState.value,
      });
    }
    await this.changeDistrictToEnabledModal();
  }

  public async changeDistrictToEnabledModal() {
    const selectCountry = Metadata.getInstance<Select>('formAddAddress-AddCountry');
    const selectState = Metadata.getInstance<Select>('formAddAddress-AddState');
    const selectCity = Metadata.getInstance<Select>('formAddAddress-AddCity');
    const selectDistrict = Metadata.getInstance<Select>('formAddAddress-AddDistrict');

    if (!selectCity.value) {
      selectDistrict.disabled = true;
      selectDistrict.value = '';
    } else {
      selectDistrict.disabled = false;
      selectDistrict.datasource.setFilter({
        CDPAIS: selectCountry.value,
        SGESTADO: selectState.value,
        CDMUNICIPIO: selectCity.value
      });
    }
  }

  public insertNewAddress() {
    const formAddAddress = Metadata.getInstance<Form>('formAddAddress');
    const grid = Metadata.getInstance<TekGrid>('tekGrid-tabAddress');
    const DISTRICT = Metadata.getInstance<Select>('formAddAddress-AddDistrict').value;

    if (formAddAddress.validate()) {
      LoadingService.show();
      Http.post('/insertNewAddress', {
        NRPARCNEGOCIO: this.BPController.currentRow.NRPARCNEGOCIO,
        CDTIPOENDERECO: Metadata.getInstance<Select>('formAddAddress-TypeAddress').value,
        NRCEPENDERECO: Metadata.getInstance<TextInput>('formAddAddress-AddCEP').value,
        DSENDERECO: Metadata.getInstance<TextInput>('formAddAddress-Address').value,
        NRIMOVELENDERECO: Metadata.getInstance<TextInput>('formAddAddress-AddressNumber').value,
        DSCOMPLEENDERECO: Metadata.getInstance<TextInput>('formAddAddress-AddAdditionalAddress').value,
        DSREFERENCIAENDE: Metadata.getInstance<TextInput>('formAddAddress-AddReference').value,
        CDPAIS: Metadata.getInstance<Select>('formAddAddress-AddCountry').value,
        SGESTADO: Metadata.getInstance<Select>('formAddAddress-AddState').value,
        CDMUNICIPIO: Metadata.getInstance<Select>('formAddAddress-AddCity').value,
        CDBAIRRO: (DISTRICT === null) ? null : DISTRICT.CDBAIRRO,
        CDOPERINCLUSAO: localStorage.getItem('CDOPERADOR'),
        NMBAIRROENDERECO: (DISTRICT === null) ? null : DISTRICT.NMBAIRRO,
      }).then(() => {
          this.alertMessages('Endereço adicionado com sucesso!', 'success');
          this.closeModalAddAddress();
          grid.datasource.get();
          LoadingService.hide();
      }).catch((error: any) => {
        Utilities.showErrorMessage(error);
        LoadingService.hide();
      });
    } else {
      this.alertMessages('Por favor, preencha todos os campos obrigatórios(*)!', 'error');
    }
  }

  // Funções do Modal de Edição -----------------------------------------------------------------------------------

  public async openModalEditionAddress({ row }: any) {
    this.editionAddress = await ModalService.createFromJson('BusinessPartner/Modal/modal-edition-address', true);
    this.NRENDERECOPARC = row.NRENDERECOPARC;
    await this.editionAddress.show();
  }

  public closeModalEditionAddress() {
    this.editionAddress.hide();
    this.editionAddress?.destroy();
  }

  public async changeStateToEnabledModalEdition() {
    const selectCountry = Metadata.getInstance<Select>('formEditionAddress-AddCountry');
    const selectState = Metadata.getInstance<Select>('formEditionAddress-AddState');

    if (!selectCountry.value) {
      selectState.disabled = true;
      selectState.value = '';
    } else {
      selectState.disabled = false;
      await selectState.datasource.setFilter({
        CDPAIS: selectCountry.value,
      });
    }
    await this.changeCityToEnabledModalEdition();
  }

  public async changeCityToEnabledModalEdition() {
    const selectCountry = Metadata.getInstance<Select>('formEditionAddress-AddCountry');
    const selectState = Metadata.getInstance<Select>('formEditionAddress-AddState');
    const selectCity = Metadata.getInstance<Select>('formEditionAddress-AddCity');

    if (!selectState.value) {
      selectCity.disabled = true;
      selectCity.value = '';
    } else {
      selectCity.disabled = false;
      selectCity.datasource.setFilter({
        CDPAIS: selectCountry.value,
        SGESTADO: selectState.value,
      });
    }
    await this.changeDistrictToEnabledModalEdition();
  }

  public async changeDistrictToEnabledModalEdition() {
    const selectCountry = Metadata.getInstance<Select>('formEditionAddress-AddCountry');
    const selectState = Metadata.getInstance<Select>('formEditionAddress-AddState');
    const selectCity = Metadata.getInstance<Select>('formEditionAddress-AddCity');
    const selectDistrict = Metadata.getInstance<Select>('formEditionAddress-AddDistrict');

    if (!selectCity.value) {
      selectDistrict.disabled = true;
      selectDistrict.value = '';
    } else {
      selectDistrict.disabled = false;
      selectDistrict.datasource.setFilter({
        CDPAIS: selectCountry.value,
        SGESTADO: selectState.value,
        CDMUNICIPIO: selectCity.value
      });
    }
  }

  public async mountInformationAddress(){
    const NRENDERECOPARC = this.NRENDERECOPARC;
    const NAMEOFCORPORATE = this.BPController.NAMEOFCORPORATE;
    let response1: any;
    LoadingService.show();
   await Http.post('/getSpecificAddress', { NRENDERECOPARC }).then((response: IDictionary) => {
      response1 = response.data.dataset.addressFound;
      LoadingService.hide();
    }).catch((error: any) => {
      Utilities.showErrorMessage(error);
      LoadingService.hide();
    });;
    response1.forEach(async(data: IDictionary) => {
      Metadata.getInstance<Text>('nameOfCorporate-editionAddress').text = NAMEOFCORPORATE;
      Metadata.getInstance<Text>('NRENDERECOPARC').text = NRENDERECOPARC;
      Metadata.getInstance<Select>('formEditionAddress-TypeAddress').setValue(data.CDTIPOENDERECO);
      Metadata.getInstance<TextInput>('formEditionAddress-AddCEP').value = data.NRCEPENDERECO;
      Metadata.getInstance<TextInput>('formEditionAddress-Address').value = data.DSENDERECO;
      Metadata.getInstance<TextInput>('formEditionAddress-AddressNumber').value = data.NRIMOVELENDERECO;
      Metadata.getInstance<TextInput>('formEditionAddress-AddAdditionalAddress').value = data.DSCOMPLEENDERECO;
      Metadata.getInstance<TextInput>('formEditionAddress-AddReference').value = data.DSREFERENCIAENDE;
      await Metadata.getInstance<Select>('formEditionAddress-AddCountry').setValue(data.CDPAIS);
      await Metadata.getInstance<Select>('formEditionAddress-AddState').setValue(data.SGESTADO);
      await Metadata.getInstance<Select>('formEditionAddress-AddCity').setValue(data.CDMUNICIPIO);
      await Metadata.getInstance<Select>('formEditionAddress-AddDistrict').setValue(data.CDBAIRRO);
    });
    await this.changeStateToEnabledModalEdition();
  }

  public updateAddress() {
    const formEditAddress = Metadata.getInstance<Form>('formEditionAddress');
    const grid = Metadata.getInstance<TekGrid>('tekGrid-tabAddress');
    const DISTRICT = Metadata.getInstance<Select>('formEditionAddress-AddDistrict').value;

    if (formEditAddress.validate()) {
      LoadingService.show();
      Http.post('/updateAddress', {
        NRENDERECOPARC: Metadata.getInstance<Text>('NRENDERECOPARC').text,
        NRPARCNEGOCIO: this.BPController.currentRow.NRPARCNEGOCIO,
        CDTIPOENDERECO: Metadata.getInstance<Select>('formEditionAddress-TypeAddress').value,
        NRCEPENDERECO: Metadata.getInstance<TextInput>('formEditionAddress-AddCEP').value,
        DSENDERECO: Metadata.getInstance<TextInput>('formEditionAddress-Address').value,
        NRIMOVELENDERECO: Metadata.getInstance<TextInput>('formEditionAddress-AddressNumber').value,
        DSCOMPLEENDERECO: Metadata.getInstance<TextInput>('formEditionAddress-AddAdditionalAddress').value,
        DSREFERENCIAENDE: Metadata.getInstance<TextInput>('formEditionAddress-AddReference').value,
        CDPAIS: Metadata.getInstance<Select>('formEditionAddress-AddCountry').value,
        SGESTADO: Metadata.getInstance<Select>('formEditionAddress-AddState').value,
        CDMUNICIPIO: Metadata.getInstance<Select>('formEditionAddress-AddCity').value,
        CDBAIRRO: (DISTRICT === null) ? null : DISTRICT.CDBAIRRO,
        CDOPERULTATU: localStorage.getItem('CDOPERADOR'),
        NMBAIRROENDERECO: (DISTRICT === null) ? null : DISTRICT.NMBAIRRO,
      }).then(() => {
          this.alertMessages('Endereço alterado com sucesso!', 'success');
          this.closeModalEditionAddress();
          grid.datasource.get();
          LoadingService.hide();
      }).catch((error: any) => {
        Utilities.showErrorMessage(error);
        LoadingService.hide();
      });;
    } else {
      this.alertMessages('Por favor, preencha todos os campos obrigatórios(*)!', 'error');
    }
  }
}
